import time
from random import random, randint

import wrapt
from rest_framework.response import Response

from .settings import DRF_CHAOS_ENABLED


def chaos(rate):
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        if random() >= rate and DRF_CHAOS_ENABLED:
            time.sleep(randint(0, 3))
            return Response(status=randint(300, 599))
        else:
            return wrapped(*args, **kwargs)

    return wrapper


def delay(rate, milliseconds):
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        if random() <= rate and DRF_CHAOS_ENABLED:
            time.sleep(milliseconds / float(1000))
        return wrapped(*args, **kwargs)

    return wrapper


def error(rate, status):
    @wrapt.decorator
    def wrapper(wrapped, instance, args, kwargs):
        if random() <= rate and DRF_CHAOS_ENABLED:
            return Response(status=status)
        else:
            return wrapped(*args, **kwargs)

    return wrapper






import time
from random import random, randint, choice

from django.http import HttpResponse
from django.http.response import REASON_PHRASES

from .settings import DRF_CHAOS_ENABLED


class ChaosMiddleware(object):
    def process_response(self, request, response):
        if random() >= 0.5 and DRF_CHAOS_ENABLED:
            time.sleep(randint(0, 3))
            response = HttpResponse()
            status_code = choice(REASON_PHRASES.keys())
            response.status_code = status_code
            response.reason_phrase = REASON_PHRASES.get(
                status_code,
                'UNKNOWN STATUS CODE'
            )
            response.content = "drf-chaos: {}".format(
                response.reason_phrase
            )
        return response
